"""The select commands module.

These commands are used in the following models:
DPO2K, DPO2KB, MSO2K, MSO2KB

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:
    ```
    - SELect {ON|OFF}
    - SELect:BUS<x> {ON|OFF|<NR1>}
    - SELect:BUS<x>?
    - SELect:CH<x> {ON|OFF|<NR1>}
    - SELect:CH<x>?
    - SELect:CONTROl {CH<x>|MATH|BUS<x>}
    - SELect:CONTROl?
    - SELect:D<x> {ON|OFF|<NR1>}
    - SELect:D<x>?
    - SELect:MATH1 {ON|OFF|<NR1>}
    - SELect:MATH1?
    - SELect:REF<x> {ON|OFF|<NR1>}
    - SELect:REF<x>?
    ```
"""

from typing import Dict, Optional, TYPE_CHECKING

from ..helpers import (
    DefaultDictPassKeyToFactory,
    SCPICmdRead,
    SCPICmdWrite,
    ValidatedChannel,
    ValidatedDigitalBit,
    ValidatedDynamicNumberCmd,
)

if TYPE_CHECKING:
    from tm_devices.driver_mixins.device_control.pi_control import PIControl


class SelectRefItem(ValidatedDynamicNumberCmd, SCPICmdWrite, SCPICmdRead):
    """The ``SELect:REF<x>`` command.

    Description:
        - Turns on and off the display of the reference waveform <x>. The <x > variable represents
          the reference channel number, which can be 1-4. The query returns whether the channel is
          on or off.

    Usage:
        - Using the ``.query()`` method will send the ``SELect:REF<x>?`` query.
        - Using the ``.verify(value)`` method will send the ``SELect:REF<x>?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``SELect:REF<x> value`` command.

    SCPI Syntax:
        ```
        - SELect:REF<x> {ON|OFF|<NR1>}
        - SELect:REF<x>?
        ```

    Info:
        - ``ON`` turns on the display of the specified waveform. This waveform also becomes the
          selected waveform.
        - ``OFF`` turns off the display of the specified waveform.
        - ``<NR1>`` = 0 turns off the display of the specified waveform; any other value turns on
          the display of the specified waveform.
    """


class SelectMath1(SCPICmdWrite, SCPICmdRead):
    """The ``SELect:MATH1`` command.

    Description:
        - Turns on and off the display of the math waveform. The query returns whether the math
          waveform is on or off but does not indicate whether it is the selected waveform.

    Usage:
        - Using the ``.query()`` method will send the ``SELect:MATH1?`` query.
        - Using the ``.verify(value)`` method will send the ``SELect:MATH1?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``SELect:MATH1 value`` command.

    SCPI Syntax:
        ```
        - SELect:MATH1 {ON|OFF|<NR1>}
        - SELect:MATH1?
        ```

    Info:
        - ``ON`` turns on the display of the specified waveform. This waveform also becomes the
          selected waveform.
        - ``OFF`` turns off the display of the specified waveform.
        - ``<NR1>`` = 0 turns off the display of the specified waveform; any other value turns on
          the display of the specified waveform.
    """


class SelectDigitalBit(ValidatedDigitalBit, SCPICmdWrite, SCPICmdRead):
    """The ``SELect:D<x>`` command.

    Description:
        - Turns on the display of the digital channel <x> and resets the acquisition. <x > is the
          channel number, which can be 0-15. The query returns whether the channel is on or off but
          does not indicate whether it is the selected waveform.

    Usage:
        - Using the ``.query()`` method will send the ``SELect:D<x>?`` query.
        - Using the ``.verify(value)`` method will send the ``SELect:D<x>?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``SELect:D<x> value`` command.

    SCPI Syntax:
        ```
        - SELect:D<x> {ON|OFF|<NR1>}
        - SELect:D<x>?
        ```

    Info:
        - ``ON`` turns on the display of the specified waveform. This waveform also becomes the
          selected waveform.
        - ``OFF`` turns off the display of the specified waveform.
        - ``<NR1>`` = 0 turns off the display of the specified waveform; any other value turns on
          the display of the specified waveform.
    """


class SelectControl(SCPICmdWrite, SCPICmdRead):
    """The ``SELect:CONTROl`` command.

    Description:
        - Sets or returns the waveform that is the recipient of future channel-related commands, for
          example, the cursor commands. The command form also performs the equivalent of a
          ``SELECT:CHX ON`` command, as well as the Math, Reference, and Bus variations of that
          command.

    Usage:
        - Using the ``.query()`` method will send the ``SELect:CONTROl?`` query.
        - Using the ``.verify(value)`` method will send the ``SELect:CONTROl?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``SELect:CONTROl value`` command.

    SCPI Syntax:
        ```
        - SELect:CONTROl {CH<x>|MATH|BUS<x>}
        - SELect:CONTROl?
        ```

    Info:
        - ``CH<x>`` specifies a channel waveform as the waveform affected by the front-panel
          controls. <x> is the channel number.
        - ``MATH`` specifies the math waveform as the waveform that is affected by the front-panel
          controls.
        - ``BUS<x>`` specifies a bus waveform as the waveform affected by the front-panel controls.
          <x> specifies the bus number.
    """


class SelectChannel(ValidatedChannel, SCPICmdWrite, SCPICmdRead):
    """The ``SELect:CH<x>`` command.

    Description:
        - This command sets or queries the displayed state of the specified channel waveform. The x
          can be channel 1 through 4.

    Usage:
        - Using the ``.query()`` method will send the ``SELect:CH<x>?`` query.
        - Using the ``.verify(value)`` method will send the ``SELect:CH<x>?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``SELect:CH<x> value`` command.

    SCPI Syntax:
        ```
        - SELect:CH<x> {ON|OFF|<NR1>}
        - SELect:CH<x>?
        ```

    Info:
        - ``<NR1> = 0`` turns off the display of the specified channel waveform; any other value
          turns on the display of the specified waveform.
        - ``OFF`` turns off the display of the indicated channel waveform.
        - ``ON`` displays the indicated channel waveform.
    """


class SelectBusItem(ValidatedDynamicNumberCmd, SCPICmdWrite, SCPICmdRead):
    """The ``SELect:BUS<x>`` command.

    Description:
        - This command turns on and off the display of the waveform for <x>, where x is the bus
          number. The query returns whether the channel is on or off but does not indicate whether
          it is the selected waveform.

    Usage:
        - Using the ``.query()`` method will send the ``SELect:BUS<x>?`` query.
        - Using the ``.verify(value)`` method will send the ``SELect:BUS<x>?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``SELect:BUS<x> value`` command.

    SCPI Syntax:
        ```
        - SELect:BUS<x> {ON|OFF|<NR1>}
        - SELect:BUS<x>?
        ```
    """


class Select(SCPICmdWrite, SCPICmdRead):
    """The ``SELect`` command.

    Description:
        - Sets or returns the selected waveform display (controlled by the front-panel) on or off.

    Usage:
        - Using the ``.write(value)`` method will send the ``SELect value`` command.

    SCPI Syntax:
        ```
        - SELect {ON|OFF}
        ```

    Info:
        - ``ON`` turns the selected waveform display on.
        - ``OFF`` turns the selected waveform display off.

    Properties:
        - ``.bus``: The ``SELect:BUS<x>`` command.
        - ``.ch``: The ``SELect:CH<x>`` command.
        - ``.control``: The ``SELect:CONTROl`` command.
        - ``.d``: The ``SELect:D<x>`` command.
        - ``.math1``: The ``SELect:MATH1`` command.
        - ``.ref``: The ``SELect:REF<x>`` command.
    """

    def __init__(self, device: Optional["PIControl"] = None, cmd_syntax: str = "SELect") -> None:
        super().__init__(device, cmd_syntax)
        self._bus: Dict[int, SelectBusItem] = DefaultDictPassKeyToFactory(
            lambda x: SelectBusItem(device, f"{self._cmd_syntax}:BUS{x}")
        )
        self._ch: Dict[int, SelectChannel] = DefaultDictPassKeyToFactory(
            lambda x: SelectChannel(device, f"{self._cmd_syntax}:CH{x}")
        )
        self._control = SelectControl(device, f"{self._cmd_syntax}:CONTROl")
        self._d: Dict[int, SelectDigitalBit] = DefaultDictPassKeyToFactory(
            lambda x: SelectDigitalBit(device, f"{self._cmd_syntax}:D{x}")
        )
        self._math1 = SelectMath1(device, f"{self._cmd_syntax}:MATH1")
        self._ref: Dict[int, SelectRefItem] = DefaultDictPassKeyToFactory(
            lambda x: SelectRefItem(device, f"{self._cmd_syntax}:REF{x}")
        )

    @property
    def bus(self) -> Dict[int, SelectBusItem]:
        """Return the ``SELect:BUS<x>`` command.

        Description:
            - This command turns on and off the display of the waveform for <x>, where x is the bus
              number. The query returns whether the channel is on or off but does not indicate
              whether it is the selected waveform.

        Usage:
            - Using the ``.query()`` method will send the ``SELect:BUS<x>?`` query.
            - Using the ``.verify(value)`` method will send the ``SELect:BUS<x>?`` query and raise
              an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``SELect:BUS<x> value`` command.

        SCPI Syntax:
            ```
            - SELect:BUS<x> {ON|OFF|<NR1>}
            - SELect:BUS<x>?
            ```
        """
        return self._bus

    @property
    def ch(self) -> Dict[int, SelectChannel]:
        """Return the ``SELect:CH<x>`` command.

        Description:
            - This command sets or queries the displayed state of the specified channel waveform.
              The x can be channel 1 through 4.

        Usage:
            - Using the ``.query()`` method will send the ``SELect:CH<x>?`` query.
            - Using the ``.verify(value)`` method will send the ``SELect:CH<x>?`` query and raise an
              AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``SELect:CH<x> value`` command.

        SCPI Syntax:
            ```
            - SELect:CH<x> {ON|OFF|<NR1>}
            - SELect:CH<x>?
            ```

        Info:
            - ``<NR1> = 0`` turns off the display of the specified channel waveform; any other value
              turns on the display of the specified waveform.
            - ``OFF`` turns off the display of the indicated channel waveform.
            - ``ON`` displays the indicated channel waveform.
        """
        return self._ch

    @property
    def control(self) -> SelectControl:
        """Return the ``SELect:CONTROl`` command.

        Description:
            - Sets or returns the waveform that is the recipient of future channel-related commands,
              for example, the cursor commands. The command form also performs the equivalent of a
              ``SELECT:CHX ON`` command, as well as the Math, Reference, and Bus variations of that
              command.

        Usage:
            - Using the ``.query()`` method will send the ``SELect:CONTROl?`` query.
            - Using the ``.verify(value)`` method will send the ``SELect:CONTROl?`` query and raise
              an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``SELect:CONTROl value`` command.

        SCPI Syntax:
            ```
            - SELect:CONTROl {CH<x>|MATH|BUS<x>}
            - SELect:CONTROl?
            ```

        Info:
            - ``CH<x>`` specifies a channel waveform as the waveform affected by the front-panel
              controls. <x> is the channel number.
            - ``MATH`` specifies the math waveform as the waveform that is affected by the
              front-panel controls.
            - ``BUS<x>`` specifies a bus waveform as the waveform affected by the front-panel
              controls. <x> specifies the bus number.
        """
        return self._control

    @property
    def d(self) -> Dict[int, SelectDigitalBit]:
        """Return the ``SELect:D<x>`` command.

        Description:
            - Turns on the display of the digital channel <x> and resets the acquisition. <x > is
              the channel number, which can be 0-15. The query returns whether the channel is on or
              off but does not indicate whether it is the selected waveform.

        Usage:
            - Using the ``.query()`` method will send the ``SELect:D<x>?`` query.
            - Using the ``.verify(value)`` method will send the ``SELect:D<x>?`` query and raise an
              AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``SELect:D<x> value`` command.

        SCPI Syntax:
            ```
            - SELect:D<x> {ON|OFF|<NR1>}
            - SELect:D<x>?
            ```

        Info:
            - ``ON`` turns on the display of the specified waveform. This waveform also becomes the
              selected waveform.
            - ``OFF`` turns off the display of the specified waveform.
            - ``<NR1>`` = 0 turns off the display of the specified waveform; any other value turns
              on the display of the specified waveform.
        """
        return self._d

    @property
    def math1(self) -> SelectMath1:
        """Return the ``SELect:MATH1`` command.

        Description:
            - Turns on and off the display of the math waveform. The query returns whether the math
              waveform is on or off but does not indicate whether it is the selected waveform.

        Usage:
            - Using the ``.query()`` method will send the ``SELect:MATH1?`` query.
            - Using the ``.verify(value)`` method will send the ``SELect:MATH1?`` query and raise an
              AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``SELect:MATH1 value`` command.

        SCPI Syntax:
            ```
            - SELect:MATH1 {ON|OFF|<NR1>}
            - SELect:MATH1?
            ```

        Info:
            - ``ON`` turns on the display of the specified waveform. This waveform also becomes the
              selected waveform.
            - ``OFF`` turns off the display of the specified waveform.
            - ``<NR1>`` = 0 turns off the display of the specified waveform; any other value turns
              on the display of the specified waveform.
        """
        return self._math1

    @property
    def ref(self) -> Dict[int, SelectRefItem]:
        """Return the ``SELect:REF<x>`` command.

        Description:
            - Turns on and off the display of the reference waveform <x>. The <x > variable
              represents the reference channel number, which can be 1-4. The query returns whether
              the channel is on or off.

        Usage:
            - Using the ``.query()`` method will send the ``SELect:REF<x>?`` query.
            - Using the ``.verify(value)`` method will send the ``SELect:REF<x>?`` query and raise
              an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``SELect:REF<x> value`` command.

        SCPI Syntax:
            ```
            - SELect:REF<x> {ON|OFF|<NR1>}
            - SELect:REF<x>?
            ```

        Info:
            - ``ON`` turns on the display of the specified waveform. This waveform also becomes the
              selected waveform.
            - ``OFF`` turns off the display of the specified waveform.
            - ``<NR1>`` = 0 turns off the display of the specified waveform; any other value turns
              on the display of the specified waveform.
        """
        return self._ref



"""The display commands module.

These commands are used in the following models:
AWG5200, AWG70KA, AWG70KB

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:
    ```
    - DISPlay:PLOT:STATe {ON|OFF|1|0}
    - DISPlay:PLOT:STATe?
    ```
"""

from typing import Optional, TYPE_CHECKING

from ..helpers import SCPICmdRead, SCPICmdWrite

if TYPE_CHECKING:
    from tm_devices.driver_mixins.device_control.pi_control import PIControl


class DisplayPlotState(SCPICmdWrite, SCPICmdRead):
    """The ``DISPlay:PLOT:STATe`` command.

    Description:
        - This command minimizes or restores the plot's display area on the Home screen's channel
          window of the AWG. This command only minimizes or restores the display area; it does not
          close the window. Plots in the Function generator window are not affected.

    Usage:
        - Using the ``.query()`` method will send the ``DISPlay:PLOT:STATe?`` query.
        - Using the ``.verify(value)`` method will send the ``DISPlay:PLOT:STATe?`` query and raise
          an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``DISPlay:PLOT:STATe value`` command.

    SCPI Syntax:
        ```
        - DISPlay:PLOT:STATe {ON|OFF|1|0}
        - DISPlay:PLOT:STATe?
        ```

    Info:
        - ``*RST`` sets this to 1.
    """


class DisplayPlot(SCPICmdRead):
    """The ``DISPlay:PLOT`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``DISPlay:PLOT?`` query.
        - Using the ``.verify(value)`` method will send the ``DISPlay:PLOT?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.state``: The ``DISPlay:PLOT:STATe`` command.
    """

    def __init__(self, device: Optional["PIControl"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._state = DisplayPlotState(device, f"{self._cmd_syntax}:STATe")

    @property
    def state(self) -> DisplayPlotState:
        """Return the ``DISPlay:PLOT:STATe`` command.

        Description:
            - This command minimizes or restores the plot's display area on the Home screen's
              channel window of the AWG. This command only minimizes or restores the display area;
              it does not close the window. Plots in the Function generator window are not affected.

        Usage:
            - Using the ``.query()`` method will send the ``DISPlay:PLOT:STATe?`` query.
            - Using the ``.verify(value)`` method will send the ``DISPlay:PLOT:STATe?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``DISPlay:PLOT:STATe value`` command.

        SCPI Syntax:
            ```
            - DISPlay:PLOT:STATe {ON|OFF|1|0}
            - DISPlay:PLOT:STATe?
            ```

        Info:
            - ``*RST`` sets this to 1.
        """
        return self._state


class Display(SCPICmdRead):
    """The ``DISPlay`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``DISPlay?`` query.
        - Using the ``.verify(value)`` method will send the ``DISPlay?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.plot``: The ``DISPlay:PLOT`` command tree.
    """

    def __init__(self, device: Optional["PIControl"] = None, cmd_syntax: str = "DISPlay") -> None:
        super().__init__(device, cmd_syntax)
        self._plot = DisplayPlot(device, f"{self._cmd_syntax}:PLOT")

    @property
    def plot(self) -> DisplayPlot:
        """Return the ``DISPlay:PLOT`` command tree.

        Usage:
            - Using the ``.query()`` method will send the ``DISPlay:PLOT?`` query.
            - Using the ``.verify(value)`` method will send the ``DISPlay:PLOT?`` query and raise an
              AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.state``: The ``DISPlay:PLOT:STATe`` command.
        """
        return self._plot



"""MP5103 device driver module."""

from tm_devices.commands import MP5103Mixin
from tm_devices.drivers.mainframes.mp5xxx.mp5xxx import MP5xxx


class MP5103(MP5103Mixin, MP5xxx):
    """MP5103 device driver."""



# pyright: reportConstantRedefinition=none
"""The upgrade commands module.

These commands are used in the following models:
SMU2461

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Attributes and Functions:
    ```
    - upgrade.previous()
    - upgrade.unit()
    ```
"""

from typing import Optional, TYPE_CHECKING

from ..helpers import BaseTSPCmd, NoDeviceProvidedError

if TYPE_CHECKING:
    from tm_devices.driver_mixins.device_control.tsp_control import TSPControl


class Upgrade(BaseTSPCmd):
    """The ``upgrade`` command tree.

    Properties and methods:
        - ``.previous()``: The ``upgrade.previous()`` function.
        - ``.unit()``: The ``upgrade.unit()`` function.
    """

    def __init__(self, device: Optional["TSPControl"] = None, cmd_syntax: str = "upgrade") -> None:
        super().__init__(device, cmd_syntax)

    def previous(self) -> str:
        """Run the ``upgrade.previous()`` function.

        Description:
            - This function returns to a previous version of the 2461 firmware.

        TSP Syntax:
            ```
            - upgrade.previous()
            ```

        Returns:
            The result of the function call.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.previous())"
            )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to run the ``{self._cmd_syntax}.previous()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def unit(self) -> None:
        """Run the ``upgrade.unit()`` function.

        Description:
            - This function upgrades the 2461 firmware.

        TSP Syntax:
            ```
            - upgrade.unit()
            ```

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            self._device.write(  # type: ignore[union-attr]
                f"{self._cmd_syntax}.unit()"
            )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to run the ``{self._cmd_syntax}.unit()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error



"""A read-only version of functools.cached_property."""

import contextlib

from functools import cached_property
from typing import TypeVar

_T = TypeVar("_T")


class ReadOnlyCachedProperty(cached_property[_T]):
    """An implementation of cached_property that is read-only.

    Notes:
        In order for the PyCharm IDE to properly provide auto-complete hints, this class must be
        imported in the following way:

        ``from tm_devices.helpers import ReadOnlyCachedProperty as cached_property``.

    Examples:
        >>> from tm_devices.helpers import ReadOnlyCachedProperty
        >>> class ClassWithReadOnlyCachedProperty:
        ...     @ReadOnlyCachedProperty
        ...     def c(self) -> str:
        ...         return "cached value"
    """

    def __set__(self, instance: object, value: _T) -> None:
        """Raise an AttributeError when trying to set the property since it is read-only.

        Raises:
            AttributeError: Indicates that the property is read-only.
        """
        msg = f"{self.attrname} is a read-only attribute"
        raise AttributeError(msg)

    def __delete__(self, instance: object) -> None:
        """Implement the __delete__ method to enable resetting the cache."""
        cache = instance.__dict__
        # Delete the attribute from the cache, ignoring KeyErrors since that just means the
        # attribute already doesn't exist and therefore doesn't need to be deleted.
        with contextlib.suppress(KeyError):
            del cache[self.attrname]  # pyright: ignore[reportArgumentType]

    @property
    def __isabstractmethod__(self) -> bool:
        """Provide a way to check if the decorated method is an abstract method."""
        return getattr(self.func, "__isabstractmethod__", False)
